var EXPORTED_SYMBOLS = [ "fburl" ];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://calendar/modules/calUtils.jsm");
Cu.import("resource:///modules/mailServices.js");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

if ("undefined" == typeof(fburl)) {
    var fburl = {};
};

const calIFreeBusyInterval = Ci.calIFreeBusyInterval;
const calIDateTime = Ci.calIDateTime;

fburl.fbUrlProvider = function(cal) {
    this.cal = cal;
    this._query = null;
    this._context = null;
    this._result = null;
    this._searched = false;
    this._error = null;
    this._listener = null;
    this._url = null;
    this._fb = null;
};

fburl.fbUrlProvider.prototype = {
    classID: Components.ID("{11291d94-b457-4322-bfba-ae9df4b6a3c1}"),

    getFreeBusyIntervals: function (aCalId, aRangeStart, aRangeEnd, aBusyTypes, aListener) {

        var start = this.cal.createDateTime("20140923T100000Z");
        var end = this.cal.createDateTime("20140923T120000Z");

        var periods = [];
        var interval = new this.cal.FreeBusyInterval(
                                        aCalId,
                                        calIFreeBusyInterval.BUSY, // fbType
                                        start, end);
        periods.push(interval);
        aListener.onResult(null, periods);
    },

    getLDAPAddressBooks: function () {

        var abManager = Cc["@mozilla.org/abmanager;1"].getService(Ci.nsIAbManager);
        var allAddressBooks = abManager.directories;
        var books = [];

        while (allAddressBooks.hasMoreElements()) {
            var addressBook = allAddressBooks.getNext().QueryInterface(Ci.nsIAbDirectory);
            if (addressBook instanceof Ci.nsIAbLDAPDirectory) {
                books.push(addressBook);
            }
        }

        return books;
    },

    startSearch: function (aCalId) {

        var service = Cc["@mozilla.org/network/ldap-service;1"].getService(Ci.nsILDAPService);
        var books = this.getLDAPAddressBooks();

        this._result = null;
        aCalId = aCalId.replace(/^mailto:/i, "");

        for each (var book in books) {
            var ldap = book.QueryInterface(Ci.nsIAbLDAPDirectory);
            var attrmap = Cc["@mozilla.org/addressbook/ldap-attribute-map;1"].createInstance(Ci.nsIAbLDAPAttributeMap);
            var args = Cc["@mozilla.org/addressbook/directory/query-arguments;1"].createInstance(Ci.nsIAbDirectoryQueryArguments);

            attrmap.setAttributeList("PrimaryEmail", "mail", true);
            attrmap.setAttributeList("FBURL", "calFbUrl", true);
            args.filter = service.createFilter(aCalId.length * 2, "(%a=%v)", "", "", "mail", aCalId);
            args.typeSpecificArg = attrmap;
            args.querySubDirectories = true;

            this._query = Cc["@mozilla.org/addressbook/ldap-directory-query;1"].createInstance(Ci.nsIAbDirectoryQuery);
            this._context = this._query.doQuery(ldap, args, this, ldap.maxHits, 0);
            this._searched = true;

            // NOTE: Do only one query for now.

            break;
        }
    },

    stopSearch: function stopSearch() {
        if (this._context) {
            this._query.stopQuery(this._context);
            this._context = null;
            this._listener = null;
        }
    },

    onSearchFinished: function (aResult, aErrorMsg) {
        this._context = null;
        this._error = aErrorMsg;
    },

    onSearchFoundCard: function (aCard) {

        this._result = aCard;

        // Obtain the URL from the card.

        this._url = aCard.getProperty("FBURL", "");

        // Retrieve the resource and parse it.

        var parser = Cc["@mozilla.org/calendar/ics-service;1"].getService(Ci.calIICSService);
        //var parser = Cc["@mozilla.org/calendar/ics-parser;1"].createInstance(Ci.calIIcsParser);
        var service = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        var stream = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
        var channel = service.newChannelFromURI(service.newURI(this._url, null, null));

        stream.init(channel.open());
        this._fb = parser.parseICS(stream.read(-1), null).getFirstSubcomponent("VFREEBUSY");
    },

    QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver, Ci.nsIAbDirSearchListener])
};

fburl.initProvider = function() {
    cal.getFreeBusyService().addProvider(new fburl.fbUrlProvider(cal));
}

fburl.test = function(aCalId) {
    var fbp = new fburl.fbUrlProvider(cal);
    fbp.startSearch(aCalId);
    return fbp;
}

const NSGetFactory = XPCOMUtils.generateNSGetFactory([fburl.fbUrlProvider]);
